# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import hashlib
import string
import random

import pooler
import tools

from osv import fields, osv
from service import security

class res_users(osv.osv):
    """Extension of the res.users object to encrypt password."""

    def __init__(self, pool, cr):
        """Redefined to mark 'salt' as a writable field."""
        res = super(res_users, self).__init__(pool, cr)
        self.SELF_WRITEABLE_FIELDS.append('salt')
        self._uid_cache.clear()
        return res

    _inherit = "res.users"

    _columns = {
        'clear_password': fields.char('Change password', size=16, help="Let it blank if you do not want to change the password"),
        # Hashed password field is called 'password' by compatibility with original res.users so that we can call super.login.
        # There is no way to set an empty password from the regular form view. This should be intentional anyway (explicit "block user access" wizard).
        'password': fields.char('Hashed password', size=40, help="ERROR: this field shall never appear in a view."),
        'salt': fields.char('Salt for hashing password', size=8),
    }

    def create(self, cr, uid, values, context=None):
        self._encrypt_password(cr, uid, values, context)
        return super(res_users, self).create(cr, uid, values, context=context)

    def write(self, cr, uid, ids, values, context=None):
        self._encrypt_password(cr, uid, values, context)
        return super(res_users, self).write(cr, uid, ids, values, context=context)

    def _encrypt_password(self, cr, uid, values, context=None):
        if values.get("clear_password"):
            values["salt"] = self._generate_salt()
            values["password"] = self._hash(values["clear_password"], values["salt"])
            del values["clear_password"] # ensures clear password is never persisted

    def _generate_salt(self):
        """Generates a random salt.
        @rtype: string"""
        sample = random.sample(string.printable, 8)
        return "".join(sample)

    def _hash(self, password, salt):
        """Returns hexadecimal string of sha1(password + salt).
        @rtype: string"""
        return hashlib.sha1(password + salt).hexdigest()

    def login(self, db, login, password):
        """Redefined to encrypt the password."""
        if not password:
            return False
        cr = pooler.get_db(db).cursor()
        cr.execute('select salt from res_users where login=%s and active', (tools.ustr(login),))
        row = cr.fetchone()
        salt = row and row[0] or ""
        cr.close()
        hashed_password = self._hash(password, salt)
        return super(res_users, self).login(db, login, hashed_password)

    def check(self, db, uid, password):
        """Redefined to encrypt the password."""
        if not password:
            return False
        cached = self._uid_cache.get(db, {}).get(uid)
        if cached:
            cached_hashed_password = cached[0]
            cached_salt = cached[1]
            hashed_password = self._hash(password, cached_salt)
            if (cached_hashed_password is not None) and cached_hashed_password == hashed_password:
                return True

        cr = pooler.get_db(db).cursor()
        cr.execute('select salt from res_users where id=%s and active=%s', (int(uid), True))
        row = cr.fetchone()
        salt = row and row[0] or ""
        hashed_password = self._hash(password, salt)
        cr.execute('select count(1) from res_users where id=%s and password=%s and active=%s', (int(uid), hashed_password, True))
        res = cr.fetchone()[0]
        cr.close()
        if not bool(res):
            raise security.ExceptionNoTb('AccessDenied')
        if res:
            if self._uid_cache.has_key(db):
                ulist = self._uid_cache[db]
                ulist[uid] = (hashed_password, salt)
            else:
                self._uid_cache[db] = {uid: (hashed_password, salt)}
        return bool(res) 

res_users()
